/* 
 * cubeSQLServerTest.java
 * Author: Rick Hofstede, University of Twente <r.j.hofstede@utwente.nl>
 */

import com.google.gson.*;
import java.util.*;

public class cubeSQLTest {
	// (Terminal) output column width in characters
	private static final int OUTPUTCOLUMNWIDTH = 15;
	
	public static void main(String[] args) {
		cubeSQLServer server = new cubeSQLServer();
		server.connect("localhost", 4430, "admin", "admin");
		
		try {
			// Change your query here
			JsonElement result = server.select("SHOW INFO;");
			
			if (result.isJsonArray()) {
				printResult((JsonArray) result);
			} else {
				System.out.println("Element is not a JsonArray! Missing implementation...");
			}
			
			server.disconnect();
		} catch (cubeSQLServerException ex) {
			// ex.printStackTrace();
		}
	}
	
	/**
	 * Prints a column-based output of the specified JsonArray to stdout.
	 * @param JsonArray JSON result array of query.
	 */
	public static void printResult (JsonArray jsonArray) {
		boolean firstRow = true; // First row in result set
		System.out.println("=== Results ===");
		
		List<String> columnNameList = new ArrayList<String>();
		List<String> lineElementsList = new ArrayList<String>();
		
		Iterator <JsonElement> objectIterator = jsonArray.iterator();
		
		while (objectIterator.hasNext()) {
			JsonElement element = objectIterator.next();
			if (element.isJsonObject()) {
				JsonObject jsonObject = (JsonObject) element;
				
				Iterator <java.util.Map.Entry <String, JsonElement>> valueIterator = jsonObject.entrySet().iterator();
				while (valueIterator.hasNext()) {
					Map.Entry pair = (Map.Entry) valueIterator.next();
					
					// Column names should only be printed before the first result row
					if (firstRow) {
						String key = (String) pair.getKey();
						columnNameList.add(truncateString(key, OUTPUTCOLUMNWIDTH));
					}

					String value = ((JsonElement) pair.getValue()).getAsString();
					lineElementsList.add(truncateString(value, OUTPUTCOLUMNWIDTH));
				}
				
				String format = "";
				for (int i = 1; i <= lineElementsList.size(); i++) {
					if (!format.equals("")) format += "| ";
					format += "%" + i + "$-" + OUTPUTCOLUMNWIDTH + "s";
				}
				
				// Convert ArrayList to Array
				String[] lineElements;
				if (firstRow) {
					lineElements = columnNameList.toArray(new String[columnNameList.size()]);
					System.out.println(String.format(format, (Object[]) lineElements));

					// Print horizontally dashed line below column names
					String dashedLine = "";
					for (int i = 0; i < OUTPUTCOLUMNWIDTH; i++) {
						dashedLine += "-";
					}
					
					for (int i = 0; i < lineElementsList.size(); i++) {
						lineElements[i] = dashedLine;
					}
					System.out.println(String.format(format, (Object[]) lineElements));
				}
				
				lineElements = lineElementsList.toArray(new String[lineElementsList.size()]);
				System.out.println(String.format(format, (Object[]) lineElements));
			} else {
				System.out.println("Element is not a JsonObject! Missing implementation...");
			}
			
			columnNameList.clear();
			lineElementsList.clear();
			firstRow = false;
		}
		
		System.out.println("===============");
	}
	
	/**
	 * Truncates a String to a given width, in case its size is longer than 'width'.
	 * @param string String to be truncated.
	 * @param width Width to which 'string' should be truncated.
	 * @return Truncated String in case its original length was longer than 'width'.
	 * Otherwise, the original string is returned.
	 */
	public static String truncateString (String string, int width) {
		String truncatedString;
		
		if (string.length() >= width) {
			
			truncatedString = string.substring(0, width - 3);
			truncatedString += "..";
		} else {
			truncatedString = string;
		}
		
		return truncatedString;
	}
}
